<?php
/**
 * PHP Version 5.3.14
 * Class NFEntrada
 * @category PHP
 * @package  Fiscal\Logic\NotaFiscal\Entrada
 * @author   paulopimenta <paulo.pimenta@teknisa.com>
 * @license  www.teknisa.com Teknisa
 * @link     http://localhost:9090/
 */
namespace Fiscal\Logic\NotaFiscal\Entrada;

use Estoque\Util\Parametros;
use Fiscal\Entities\Contapagar;
use Fiscal\Entities\Dadosadic;
use Fiscal\Entities\Incitemp;
use Fiscal\Entities\IntgItemencargo;
use Fiscal\Entities\IntgNotafiscal;
use Fiscal\Entities\IntgTrabitproimpos;
use Fiscal\Entities\Itemproimpos;
use Fiscal\Entities\Notafiscal;
use Fiscal\Entities\Trabimposnftot;
use Fiscal\Entities\Trabitproimpos;
use Fiscal\Logic\ImportacaoAF\ImportadorAF;
use Fiscal\Logic\NotaFiscal\Lancamento\IEvents;
use Fiscal\Logic\NotaFiscal\ProceduresNF;
use Geral\Entities\Empresa;
use Geral\Entities\Endeclie;
use Geral\Entities\Endefili;
use Geral\Entities\Endeforn;
use Geral\Entities\Itemencargo;
use Geral\Entities;
use Geral\Entities\Itemnotafisc;
use Geral\Entities\Itemproduto;
use Geral\Entities\Lote;
use Geral\Entities\Tipooperac;
use Geral\Logic\Produto\Produto;
use Geral\Util\Ambiente;
use Geral\Util\DateUtil;
use Geral\Util\NovoCodigo;
use Indeterminado\Entities\Aliqtempprod;
use Indeterminado\Entities\Paramforne;
use Indeterminado\Entities\Prodfornxml;
use Setforms2\Runtime;
use Setforms2\Shared\ORM\BasicEntity;
use Setforms2\Shared\Registry;

/**
 * PHP Version 5.3.14
 * Class NFEntrada
 * @category PHP
 * @package  Fiscal\Logic\NotaFiscal\Entrada
 * @author   paulopimenta <paulo.pimenta@teknisa.com>
 * @license  www.teknisa.com Teknisa
 * @link     http://localhost:9090/
 */
class NFEntrada implements IEvents
{

    private $notafiscal;

    public function __construct(Notafiscal $notafiscal)
    {
        $this->notafiscal = $notafiscal;
    }

    /**
     * Gera o novo codigo da nota fiscal.
     * @return int|string
     */
    public static function novoCodigo()
    {
        $cdfilial = Ambiente::getCdFilial();
        $nrorg    = Ambiente::getNrorg();
        return NovoCodigo::geraCodigo(
            $nrorg, 'LANCNF/FILIAL' . $cdfilial
        );
    }

    /**
     * Deleta o vencimento da nota fiscal
     * @param Notafiscal $notafiscal
     * @return void
     */
    public static function deletaVencimentoNotaFiscal(Notafiscal $notafiscal)
    {
        /** @var $contapagar Contapagar */
        $contapagar = \Fiscal\Entities\Contapagar::getObjectByWhere(
            'NRORG = ? AND CDFILIAL = ? AND NRLANCTONF = ?',
            array(
                $notafiscal->getNrorg(),
                $notafiscal->getFilial()->getCdfilial(),
                $notafiscal->getNrlanctonf())
        );

        if (is_object($contapagar)) {
            $contapagar->delete();
            \Setforms2\Shared\Registry::getEntityManager()->flush();
        }
    }

    /**
     * Gera o vencimento da nota fiscal
     * @param Notafiscal $notafiscal
     * @return bool
     */
    public static function geraVencimentoNotaFiscal(Notafiscal $notafiscal)
    {
        $contapagar = \Fiscal\Entities\Contapagar::getObjectByWhere(
            'NRORG = ? AND CDFILIAL = ? AND NRLANCTONF = ?',
            array(
                $notafiscal->getNrorg(),
                $notafiscal->getFilial()->getCdfilial(),
                $notafiscal->getNrlanctonf())
        );

        if (is_object($contapagar)) {
            return false;
        } else {
            $id = array(
                'nrorg'        => $notafiscal->getNrorg(),
                'cdtipooper'   => $notafiscal->getCdtipooper(),
                'identrsaidop' => $notafiscal->getIdentrsaidop());

            /** @var $tpoperac \Geral\Entities\Tipooperac */
            $tpoperac = \Geral\Entities\Tipooperac::getObject($id);

            if ($tpoperac->getIdutilvenc() != 'N'
                && $tpoperac->getIdgeravenc() != 'N'
            ) {
                if ($notafiscal->getIdorigdestnf() === 'F') {
                    if ($notafiscal->getVrnotafisc() === 0) {
                        return false;
                    } else {
                        ProceduresNF::procedureGeraVencimento(
                            $notafiscal->getCdfilial(),
                            $notafiscal->getNrlanctonf(), 'L'
                        );
                    }
                }
            }
        }
        return false;
    }

    public static function SalvaAutomatico($cdserieentr, $dsserieentr,
                                           $idgeractbxp, $data,
                                           $cdserieentrnfe, $cdserieentscan,
                                           $nrorg, $data, $cdoperador)
    {
        //        try {
        //            /** @var $repository  \Geral\Repositories\Serieentr*/
        //            $repository = \Geral\Entities\Serieentr::getRepository();
        //            $repository->salvaAutomatico($cdserieentr, $dsserieentr, $idgeractbxp, $data, $cdserieentrnfe, $cdserieentscan, $nrorg, $data, $cdoperador);
        //        } catch (\Exception $e) {
        //            throw $e;
        //        }
        $serieEntr = \Geral\Entities\Serieentr::findOneByCdserieentrAndNrorg(
            $cdserieentr, $nrorg
        );
        if (!$serieEntr instanceof \Geral\Entities\Serieentr) {
            $serieEntr = new \Geral\Entities\Serieentr();
            $serieEntr->setCdserieentr($cdserieentr);
            $serieEntr->setDsserieentr($dsserieentr);
            $serieEntr->setIdgeractbxp($idgeractbxp);
            $serieEntr->setDtultatusent($data);
            $serieEntr->setCdserieentrnfe($cdserieentrnfe);
            $serieEntr->setCdserieentscan($cdserieentscan);

            $serieEntr->setNrorg($nrorg);
            $serieEntr->setDtinclusao($data);
            $serieEntr->setCdoperinclusao($cdoperador);
            $serieEntr->setNrorginclusao($nrorg);
            $serieEntr->setDtultatu($data);
            $serieEntr->setCdoperultatu($cdoperador);
            $serieEntr->setNrorgultatu($nrorg);

            $serieEntr->persist();
            \Setforms2\Shared\Registry::getEntityManager()->flush();
        }
    }

    /**
     * Atualiza serie
     * @param $dsserieentr
     * @param $cdserieentr
     * @param $nrorg
     * @return void
     * @throws \Exception
     */
    public static function updateSerie($dsserieentr, $cdserieentr, $nrorg)
    {
        try {
            /** @var $repository  \Geral\Repositories\Serieentr */
            $repository = \Geral\Entities\Serieentr::getRepository();
            $repository->atualizaSerie($dsserieentr, $cdserieentr, $nrorg);
        } catch (\Exception $e) {
            throw $e;
        }
    }

    /**
     * Deleta serie
     * @param $cdserieentr
     * @param $nrorg
     * @return void
     * @throws \Exception
     */
    public static function deletaSerie($cdserieentr, $nrorg)
    {
        try {
            /** @var $serieEntr \Geral\Entities\Serieentr */
            $serieEntr = \Geral\Entities\Serieentr::findOneByCdserieentrAndNrorg($cdserieentr, $nrorg);
            $serieEntr->delete();

        } catch (\Exception $e) {
            throw $e;
        }
    }

    /**
     * Verifica se existe XML para importar os dados através do campo CHAVE_ACESSO.
     * Se existir, preenche os campos da tela e bloqueia alteração.
     *
     * @param $chaveAcesso
     * @param $nrorg
     */
    public static function verificaXMLporChaveAcesso($chaveAcesso, $nrorg)
    {
        return IntgNotafiscal::findOneByNracessonfeAndNrorg($chaveAcesso, $nrorg);
    }

    /**
     * Verifica se existe XML para importar os dados através dos campos:
     *
     * @param $fornecedor
     * @param $nf
     * @param $serie
     * @param $dataEmissao
     */
    public static function verificaXML($fornecedor, $nf, $serie, $dataEmissao)
    {
        $where = 'CDFORNECED = ? AND NRNOTAFISC = ? AND CDSERIEENTR = ? AND DTEMISSAO = ?';
        $bind  = array(
            $fornecedor,
            $nf,
            $serie,
            $dataEmissao);

        return IntgNotafiscal::getObjectByWhere($where, $bind);
    }

    public static function saveProdutos($produtos, $fornecedor, $lancamento, $itemProdutoJaSalvo, $rowsAF = null,
                                        $chaveAcesso = null)
    {
        $impostoDeletado = false;

        foreach ($produtos as $produto) {
            $produtoXML = $produto['CDPRODUTO'];

            /** @var $jaCadastrado Prodfornxml */
            $jaCadastrado = Prodfornxml::findOneByCdfornecedAndCdprodutoxml($fornecedor, $produtoXML);

            if ($jaCadastrado instanceof BasicEntity) {
                if ($jaCadastrado->getCdproduto() == $produto['CDPRODPEB']) {
                    if ($itemProdutoJaSalvo == 1) {
                        continue;
                    }

                    $itemProduto = self::saveProdutoNF($produto, $lancamento);
                    if ($rowsAF) {
                        $importadorAF = new ImportadorAF($rowsAF);
                        $importadorAF->insereItemprodutoAF($itemProduto);
                    }

                    self::setImpostosProduto($itemProduto);
                    if ($chaveAcesso) {
                        self::saveImpostosXML($itemProduto, $produto, $chaveAcesso);
                    }
                    continue;
                }

                $manager = Registry::getEntityManager();
                $jaCadastrado->delete();
                $manager->flush();
            }

            if ($itemProdutoJaSalvo == 1) {
                self::deleteProduto_e_Imposto($jaCadastrado->getCdproduto(), $lancamento);
                $impostoDeletado = true;
            }

            self::saveProdutoPebbian($produto, $fornecedor, $produtoXML);
            $itemProduto = self::saveProdutoNF($produto, $lancamento);
            if ($rowsAF) {
                $importadorAF = new ImportadorAF($rowsAF);
                $importadorAF->insereItemprodutoAF($itemProduto);
            }
            self::setImpostosProduto($itemProduto);
            if ($chaveAcesso) {
                self::saveImpostosXML($itemProduto, $produto, $chaveAcesso);
            }
        }
        return $impostoDeletado;
    }

    public static function saveProdutoNF($produto, $lancamento)
    {
        $cdalmoxarife = Parametros::getAlmoxarifado();
        $cdlocalestoq = Parametros::getLocalEstoque();
        $nrloteestq   = Parametros::getNumeroLote();
        $nrsublote    = ' ';

        $cdalmoxarife = !empty($cdalmoxarife) ? $cdalmoxarife : ' ';
        $cdlocalestoq = !empty($cdlocalestoq) ? $cdlocalestoq : ' ';
        $nrloteestq   = !empty($nrloteestq) ? $nrloteestq : ' ';

        $manager = Registry::getEntityManager();

        /** @var $itemProduto Entities\Itemproduto */
        $itemProduto = new Entities\Itemproduto();

        $itemProduto->setNrorg(Ambiente::getNrorg());
        $itemProduto->setDtinclusao(DateUtil::getDataAtual());
        $itemProduto->setDtultatu(DateUtil::getDataAtual());
        $itemProduto->setNrorginclusao(Ambiente::getNrorg());
        $itemProduto->setCdoperinclusao(Ambiente::getCdOperador());
        $itemProduto->setNrorgultatu(Ambiente::getNrorg());
        $itemProduto->setCdoperultatu(Ambiente::getCdOperador());
        $itemProduto->setNrlanctonf($lancamento);
        $itemProduto->setCdoperador(Ambiente::getCdOperador());
        $itemProduto->setCdalmoxarife($cdalmoxarife);
        $itemProduto->setCdlocalestoq($cdlocalestoq);
        $itemProduto->setNrloteestq($nrloteestq);
        $itemProduto->setNrsublote($nrsublote);
        $itemProduto->setNrseqitprod(self::geraProximoNRSEQITPROD(Ambiente::getCdFilial(), $lancamento));
        $itemProduto->setIdatuvrestoq('S');
        $itemProduto->setVrcontitem($produto['VRCONTITEM']);
        $itemProduto->setIdconforitem('S');
        $itemProduto->setVrbrutitem($produto['VRBRUTITEM']);
        $itemProduto->setCdfilial(Ambiente::getCdfilial());
        $itemProduto->setCdfilimovi(Ambiente::getCdfilial());
        $itemProduto->setCdproduto($produto['CDPRODPEB']);
        $itemProduto->setVrunititem($produto['VRUNITITEM']);
        $itemProduto->setVrtotaitem($produto['VRTOTAITEM']);
        $itemProduto->setQtitemprod($produto['QTITEMPROD']);
        $itemProduto->setVrdescitem($produto['VRDESCITEM']);
        $itemProduto->setCdprodemis($itemProduto->getProduto()->getCdproduto());
        $itemProduto->setDsprodemis($itemProduto->getProduto()->getNmproduto());

        $itemProduto->persist();

        $manager->flush();

        return $itemProduto;
    }

    /**
     * Gera codigo sequencial para o novo produto
     * @param $filial
     * @param $lancamento
     * @return string
     */
    public static function geraProximoNRSEQITPROD($filial, $lancamento)
    {
        $nrseqitprod = Entities\Itemproduto::getRepository()->getMaxNRSEQITPROD($filial, $lancamento) + 1;

        return str_pad($nrseqitprod, 4, '0', STR_PAD_LEFT);
    }

    public static function setImpostosProduto(Entities\Itemproduto $itemproduto)
    {
        $cdgrupofisc = ProceduresNF::exec_consulta_grupofiscal($itemproduto->getLanctonf());

        $notafiscal = Notafiscal::findOneByNrlanctonfAndNrorg(
            $itemproduto->getNrlanctonf(),
            Ambiente::getNrorg()
        );
        $impostos   = self::calculaImpostosProduto(
            $notafiscal, $itemproduto, $cdgrupofisc,
            $itemproduto->getProduto()->getCdarvprod()
        );

        foreach ($impostos as $impotemp)
            self::saveImpostoProduto(
                $itemproduto, $impotemp['VRALIQUOTA'], $impotemp['VRIMPOSTOIT'],
                $impotemp['CDIMPOSTO']
            );
    }

    public static function calculaImpostosProduto(Notafiscal $notafiscal, Itemproduto $itemproduto, $grupofiscal,
                                                  $cdarvprod, $filialOrigem = null, $lancamentoOrigem = null)
    {
        //if($notafiscal->getCdtipooper() != '03'){
        /* @var $tipooperac Tipooperac */
        $tipooperac = Tipooperac::findOneByIdentrsaidopAndCdtipooperAndNrorg(
            $notafiscal->getIdentrsaidop(),
            $notafiscal->getCdtipooper(), Ambiente::getNrorg()
        );

        if ($tipooperac->getIdoperdevonf() != 'S') {
            $vrproduto  = ($itemproduto->getVrtotaitem() - $itemproduto->getVrdescitem());
            $idinterest = self::getIdinterest($notafiscal, $tipooperac);
            ProceduresNF::callProcedureImpostoEntradaProduto(
                $notafiscal, $grupofiscal, $cdarvprod, $vrproduto,
                $idinterest
            );
            $impostosTemporarios = Incitemp::getRepository()->getImpostosTemporarios(
                $itemproduto->getCdfilial(),
                $itemproduto->getNrlanctonf(), $itemproduto->getNrorg()
            );
            self::deletaImpostoTemporario(
                $notafiscal->getCdfilial(), $notafiscal->getNrlanctonf(),
                $notafiscal->getNrorg()
            );
            return $impostosTemporarios;
        }

        return Itemproimpos::getRepository()->consultaImpostosItemNotaDevolucao(
            $filialOrigem, $lancamentoOrigem,
            $itemproduto->getCdproduto(), $itemproduto->getNrloteestq(), $itemproduto->getNrsublote(),
            $itemproduto->getVrunititem()
        );
        //}
    }

    public static function getIdinterest(Notafiscal $notafiscal, Tipooperac $tipooperac)
    {
        $idinterest = 'N';
        $nrorg      = $notafiscal->getNrorg();

        $endeEmpresa = Empresa::getRepository()->getEstadoPais(
            $notafiscal->getCdempresa(),
            $notafiscal->getCdinscesta(), $nrorg
        );

        switch ($tipooperac->getIdorigdestop()) {
        case 'F':
            $endeForn = Endeforn::getRepository()->getEstadoPais($notafiscal->getCdforneced(), $nrorg);
            if ($endeEmpresa['CDPAIS'] == $endeForn['CDPAIS'] && $endeEmpresa['SGESTADO'] != $endeForn['SGESTADO']) {
                $idinterest = 'S';
            }
            break;
        case 'C':
            $endeClie = Endeclie::getRepository()->getEstadoPais($notafiscal->getCdcliente(), $nrorg);
            if ($endeEmpresa['CDPAIS'] == $endeClie['CDPAIS'] && $endeEmpresa['SGESTADO'] != $endeClie['SGESTADO']) {
                $idinterest = 'S';
            }
            break;
        case 'L':
            $endeFili = Endefili::getRepository()->getEstadoPais($notafiscal->getCdfiliemit(), $nrorg);
            if ($endeEmpresa['CDPAIS'] == $endeFili['CDPAIS'] && $endeEmpresa['SGESTADO'] != $endeFili['SGESTADO']) {
                $idinterest = 'S';
            }
            break;
        }

        return $idinterest;
    }

    /**
     * Deleta impostos temporarios
     * @param $cdfilial
     * @param $nrlanctonf
     * @param $nrorg
     * @return void
     */
    public static function deletaImpostoTemporario($cdfilial, $nrlanctonf, $nrorg, $cdimposto = null)
    {
        if($cdimposto){
            $aliqtempprod = Aliqtempprod::findOneByNraliqtempAndNrorgAndCdimpotemp($cdfilial . $nrlanctonf, $nrorg, $cdimposto);
            if($aliqtempprod instanceof Aliqtempprod){
                $aliqtempprod->delete();
            }

            $incitemp = Incitemp::findOneByNrincitempAndNrorgAndCdimpotemp($cdfilial . $nrlanctonf, $nrorg, $cdimposto);
            if($incitemp instanceof Incitemp){
                $incitemp->delete();
            }
        }
        else{
            $aliqtempprod = Aliqtempprod::findByNraliqtempAndNrorg($cdfilial . $nrlanctonf, $nrorg);
            foreach ($aliqtempprod as $aliq)
                $aliq->delete();

            $incitemp = Incitemp::findByNrincitempAndNrorg($cdfilial . $nrlanctonf, $nrorg);
            foreach ($incitemp as $inci)
                $inci->delete();
        }

        Registry::getEntityManager()->flush();
    }

    public static function saveImpostoProduto(Entities\Itemproduto $itemproduto, $alimposto, $vrimposto, $cdimposto,
                                              $idvalorinfo = 'N')
    {
        $connection = \Setforms2\Shared\Registry::getEntityManager();
        $connection->beginTransaction();
        try {
            $alimposto = !empty($alimposto) ? $alimposto : 0;
            $vrimposto = !empty($vrimposto) ? $vrimposto : 0;

            $trabitproimpos = Trabitproimpos::getObjectByWhere(
                'CDFILIMPOIT = ? AND NRLANCIMPOIT = ? AND NRSEQIMPOIT = ? AND CDIMPOSTOIT = ? AND NRSEQITPRO = ? AND NRORG = ?',
                array(
                    $itemproduto->getCdfilial(),
                    $itemproduto->getNrlanctonf(),
                    $itemproduto->getNrseqitprod(),
                    $cdimposto,
                    '1',
                    $itemproduto->getNrorg())
            );

            if (($trabitproimpos instanceof Trabitproimpos) === false) {
                $trabitproimpos = new Trabitproimpos();

                // PK_ITEMPROIMPOS
                $trabitproimpos->setCdfilimpoit($itemproduto->getCdfilial());
                $trabitproimpos->setNrlancimpoit($itemproduto->getNrlanctonf());
                $trabitproimpos->setNrseqimpoit($itemproduto->getNrseqitprod());
                $trabitproimpos->setCdimpostoit($cdimposto);
                $trabitproimpos->setNrseqitpro('1');
                $trabitproimpos->setNrorg($itemproduto->getNrorg());

                $trabitproimpos->setCdoperinclusao(Ambiente::getCdOperador());
                $trabitproimpos->setDtinclusao(DateUtil::getDataDeString(now(), DateUtil::FORMATO_BRASILEIRO_DATAHORA));

                $trabitproimpos->setVrcontimpoit('0');
                $trabitproimpos->setVrredbcimpoit('0');
                $trabitproimpos->setVrbcalcimpoit('0');
                $trabitproimpos->setVrpdedimpoit('0');

                $trabitproimpos->setIdimpdesotrabit('N');
                $trabitproimpos->setIdmotdesotrabit('9');
                $trabitproimpos->setIddifaliqcuspti('N');
            }

            $trabitproimpos->setVrimposit($vrimposto);
            $trabitproimpos->setVraliqimpoit($alimposto);
            $trabitproimpos->setIdvalorinfo($idvalorinfo);
            $trabitproimpos->setCdoperultatu(Ambiente::getCdOperador());
            $trabitproimpos->setDtultatu(DateUtil::getDataDeString(now(), DateUtil::FORMATO_BRASILEIRO_DATAHORA));

            $trabitproimpos->persist();

            Registry::getEntityManager()->flush();
            $connection->commit();
        } catch (\Exception $e) {
            $connection->rollback();
            tk_abort($e->getMessage());
        }
    }

    public static function saveImpostosXML(Entities\Itemproduto $itemProduto, $produto, $chaveAcesso)
    {
        $impostoXML = IntgTrabitproimpos::findByNracessonfeAndNrseqimpoitAndNrorg(
            $chaveAcesso, $produto['NRSEQITPROD'],
            Ambiente::getNrorg()
        );
        /** @var $imposto IntgTrabitproimpos */
        foreach ($impostoXML as $imposto) {
            self::saveImpostoProduto(
                $itemProduto, $imposto->getVraliqitem(), $imposto->getVrimpostoit(),
                $imposto->getCdimpostoit()
            );
        }
    }

    public static function deleteProduto_e_Imposto($cdproduto, $lancamento)
    {
        $manager = Registry::getEntityManager();

        /** @var $itemProduto Entities\Itemproduto */
        $itemProduto = Entities\Itemproduto::findOneByCdfilialAndNrlanctonfAndNrorgAndCdproduto(
            Ambiente::getCdFilial(),
            $lancamento, Ambiente::getNrorg(), $cdproduto
        );

        self::deleteImpostoItemproduto($itemProduto);
        // 		/** @var $itemproimpos Itemproimpos */
        // 		$itemproimpos = Itemproimpos::getObjectByWhere(
        // 				'CDFILIAL = ? AND NRLANCTONF = ? AND NRSEQITPROD = ? AND NRITPROIMPOS = ? AND NRORG = ?',
        // 				array(
        // 						$itemProduto->getCdfilial(),
        // 						$itemProduto->getNrlanctonf(),
        // 						$itemProduto->getNrseqitprod(),
        // 						'1',
        // 						$itemProduto->getNrorg()));

        // 		$itemproimpos->delete();

        $itemProduto->delete();

        $manager->flush();
    }

    public static function deleteImpostoItemproduto(Entities\Itemproduto $itemproduto)
    {
        /** @var $itemproimpos Itemproimpos */
        $itemproimpos = Itemproimpos::getObjectsByWhere(
            'CDFILIAL = ? AND NRLANCTONF = ? AND NRSEQITPROD = ? AND NRORG = ?',
            array(
                $itemproduto->getCdfilial(),
                $itemproduto->getNrlanctonf(),
                $itemproduto->getNrseqitprod(),
                $itemproduto->getNrorg())
        );

        foreach ($itemproimpos as $imposto) {
            $manager = Registry::getEntityManager();

            $imposto->delete();

            $manager->flush();
        }
    }

    public static function saveProdutoPebbian($produto, $fornecedor, $produtoXML)
    {
        $manager = Registry::getEntityManager();

        /** @var $prodfornxml Prodfornxml */
        $prodfornxml = new Prodfornxml();

        $prodfornxml->setCdproduto($produto['CDPRODPEB']);
        $prodfornxml->setCdforneced($fornecedor);
        $prodfornxml->setCdprodutoxml($produtoXML);
        $prodfornxml->setNmprodutoxml($produto['NMPRODUTO']);
        $prodfornxml->setNrorg(Ambiente::getNrorg());
        $prodfornxml->setDtinclusao(DateUtil::getDataAtual());
        $prodfornxml->setDtultatu(DateUtil::getDataAtual());
        $prodfornxml->setNrorginclusao(Ambiente::getNrorg());
        $prodfornxml->setCdoperinclusao(Ambiente::getCdOperador());
        $prodfornxml->setNrorgultatu(Ambiente::getNrorg());
        $prodfornxml->setCdoperultatu(Ambiente::getCdOperador());

        $prodfornxml->persist();

        $manager->flush();
    }

    public static function deleteImpostoNotaTotal($cdfilial, $lancamento, $nrorg, $cdimposto = null)
    {
        if($cdimposto){
            $imposto = Trabimposnftot::findByCdfilimptotAndNrlancimptotAndNrorgAndCdimpostot(
                $cdfilial, $lancamento, $nrorg, $cdimposto
            );

            if($imposto instanceof Trabimposnftot){
                $imposto->delete();
            }
        }
        else{
            $impostos = Trabimposnftot::findByCdfilimptotAndNrlancimptotAndNrorg(
                $cdfilial, $lancamento, $nrorg
            );

            if (!empty($impostos)) {
                /** @var $imposto Trabimposnftot */
                foreach ($impostos as $imposto) {
                    $imposto->delete();
                }
            }
        }

        Registry::getEntityManager()->flush();
    }

    public static function deleteImpostoTemporarioProduto($cdfilial, $lancamento, $nrorg, $nrseqimpo, $cdimposto, $nrseqitprod)
    {
        $imposto = Trabitproimpos::getObjectByWhere('CDFILIMPOIT  = ? AND NRLANCIMPOIT = ? AND NRSEQIMPOIT  = ? AND CDIMPOSTOIT  = ? AND NRSEQITPRO   = ? AND NRORG = ?',
            array($cdfilial, $lancamento, $nrseqimpo, $cdimposto, $nrseqitprod, $nrorg)
        );

        if ($imposto instanceof Trabitproimpos) {
            /** @var $imposto Trabitproimpos */
            $imposto->delete();
            Registry::getEntityManager()->flush();
        }
    }

    public static function saveProdutoLote($cdfilial, $cdproduto)
    {
        Lote::getRepository()->insereProdutoLote($cdfilial, ' ', ' ', $cdproduto, ' ', ' ');
    }

    public static function getProximoNRSEQVENPAG($filial, $lancamento)
    {
        $nrseqvenpag = Contapagar::getRepository()->getMaxNRSEQVENPAG($filial, $lancamento) + 1;

        return str_pad($nrseqvenpag, 2, '0', STR_PAD_LEFT);
    }

    public static function getProximoNRDADOADIC($filial, $lancamento)
    {
        $nrdadoadic = Dadosadic::getRepository()->getMaxNRDADOADIC($filial, $lancamento) + 1;

        return str_pad($nrdadoadic, 3, '0', STR_PAD_LEFT);
    }

    public static function saveEncargosNF($nrorg, $filial, $nrlanctonf, $chaveAcesso)
    {
        $intg_itemencargo = IntgItemencargo::findByNracessonfeAndNrorg($chaveAcesso, $nrorg);

        foreach ($intg_itemencargo as $encargo) {
            $manager = Registry::getEntityManager();

            $itemencargo = new Itemencargo();

            $itemencargo->setCdfilial($filial);
            $itemencargo->setNrlanctonf($nrlanctonf);
            $itemencargo->setCdencargo($encargo->getCdencargo());
            $itemencargo->setVritemenca($encargo->getVritemenca());
            $itemencargo->setNrorg($nrorg);
            $itemencargo->setDtinclusao(DateUtil::getDataAtual());
            $itemencargo->setDtultatu(DateUtil::getDataAtual());
            $itemencargo->setCdoperinclusao(Ambiente::getCdOperador());
            $itemencargo->setCdoperultatu(Ambiente::getCdOperador());

            $itemencargo->persist();

            $manager->flush();
        }
    }

    public static function saveImpostosNF($nrorg, $filial, $nrlanctonf)
    {
        $impostos       = array();
        $trabitproimpos = Trabitproimpos::findByCdfilimpoitAndNrlancimpoitAndNrorg($filial, $nrlanctonf, $nrorg);

        /** @var $imposto Trabitproimpos */
        foreach ($trabitproimpos as $imposto) {
            if (empty($impostos[$imposto->getCdimpostoit()])) {
                $impostos[$imposto->getCdimpostoit()][0] = 0;
            }
            $impostos[$imposto->getCdimpostoit()][0] += $imposto->getVrimposit();
        }

        foreach ($impostos as $imposto => $valor) {
            $manager = Registry::getEntityManager();

            $trabimposnftot = new Trabimposnftot();
            $trabimposnftot->setCdfilimptot($filial);
            $trabimposnftot->setNrlancimptot($nrlanctonf);
            $trabimposnftot->setCdimpostot($imposto);
            $trabimposnftot->setVrimpostot($valor[0]);
            $trabimposnftot->setVrbcalimptot('0');
            $trabimposnftot->setNrorg($nrorg);
            $trabimposnftot->setDtinclusao(DateUtil::getDataAtual());
            $trabimposnftot->setDtultatu(DateUtil::getDataAtual());
            $trabimposnftot->setCdoperinclusao(Ambiente::getCdOperador());
            $trabimposnftot->setCdoperultatu(Ambiente::getCdOperador());
            $trabimposnftot->setIddifaliqcusptn('N');

            $trabimposnftot->persist();

            //             $imposnftot = new Imposnftot();

            //             $imposnftot->setCdfilial($filial);
            //             $imposnftot->setNrlanctonf($nrlanctonf);
            //             $imposnftot->setCdimposto($imposto);
            //             $imposnftot->setVrtotimponf($valor[0]);
            //             $imposnftot->setVrtotbcalcnf($valor[1]);
            //             $imposnftot->setNrorg($nrorg);
            //             $imposnftot->setDtinclusao(DateUtil::getDataAtual());
            //             $imposnftot->setDtultatu(DateUtil::getDataAtual());
            //             $imposnftot->setCdoperinclusao(Ambiente::getCdOperador());
            //             $imposnftot->setCdoperultatu(Ambiente::getCdOperador());

            //             $imposnftot->persist();

            $manager->flush();
        }
    }

    public static function atualizaQtrelacNforigem($cdfilial, $nrlanctonf, $nrseqitprod, $vrdiferenca, $idstatus)
    {
        ProceduresNF::callProcedureAtualizaQtrelacNforigem(
            $cdfilial, $nrlanctonf, $nrseqitprod, $vrdiferenca,
            $idstatus
        );
    }

    public static function deleteProdutosNF(Itemproduto $produto, $lancamento, $nrseqitprod)
    {
        $itemProduto = Itemproduto::findOneByNrlanctonfAndNrseqitprodAndCdfilialAndNrorg(
            $lancamento, $nrseqitprod,
            $produto->getCdfilial(), $produto->getNrorg()
        );
        $itemProduto->delete();
        Registry::getEntityManager()->flush();
    }

    public static function saveProdutosNF(Itemproduto $produto, $lancamento, $nrseqitprod = null)
    {
        if (!$nrseqitprod) {
            $itemProduto = new Itemproduto();

            $itemProduto->setNrorg(Ambiente::getNrorg());
            $itemProduto->setDtinclusao(DateUtil::getDataAtual());
            $itemProduto->setNrorginclusao(Ambiente::getNrorg());
            $itemProduto->setCdoperinclusao(Ambiente::getCdOperador());
            $itemProduto->setNrlanctonf($lancamento);
            $itemProduto->setNrseqitprod(self::geraProximoNRSEQITPROD(Ambiente::getCdFilial(), $lancamento));
            $itemProduto->setCdfilial(Ambiente::getCdFilial());
            $itemProduto->setCdfilimovi($produto->getCdfilimovi());
        } else {
            $itemProduto = Itemproduto::findOneByNrlanctonfAndNrseqitprodAndCdfilialAndNrorg(
                $lancamento, $nrseqitprod,
                $produto->getCdfilial(), $produto->getNrorg()
            );
        }

        $itemProduto->setCdoperador(Ambiente::getCdOperador());
        $itemProduto->setCdalmoxarife($produto->getCdalmoxarife() ? : ' ');
        $itemProduto->setCdlocalestoq($produto->getCdlocalestoq() ? : ' ');
        $itemProduto->setNrloteestq($produto->getNrloteestq() ? : ' ');
        $itemProduto->setNrsublote($produto->getNrsublote() ? : ' ');
        $itemProduto->setCdfilialped($produto->getCdfilialped());
        $itemProduto->setNrpedido($produto->getNrpedido());
        $itemProduto->setNritemped($produto->getNritemped());
        $itemProduto->setQtvoluitem($produto->getQtvoluitem());
        $itemProduto->setCdservfatu($produto->getCdservfatu());
        $itemProduto->setNrpedprod($produto->getNrpedprod());
        $itemProduto->setNrseqpedprod($produto->getNrseqpedprod());
        $itemProduto->setCdfiscopersn($produto->getCdfiscopersn());
        $itemProduto->setCdservfili($produto->getCdservfili());
        $itemProduto->setQtitrelacnfsubs($produto->getQtitrelacnfsubs());
        $itemProduto->setIdatuvrestoq($produto->getIdatuvrestoq());
        $itemProduto->setVrcontitem($produto->getVrcontitem());
        $itemProduto->setIdconforitem($produto->getIdconforitem());
        $itemProduto->setVrbrutitem($produto->getVrbrutitem());
        $itemProduto->setVrdesperitem($produto->getVrdesperitem());
        $itemProduto->setCdproduto($produto->getCdproduto());
        $itemProduto->setVrunititem($produto->getVrunititem());
        $itemProduto->setVrdescitem($produto->getVrdescitem());
        $itemProduto->setCdprodemis($produto->getCdprodemis());
        $itemProduto->setDsprodemis($produto->getDsprodemis());
        $itemProduto->setCdfisopeitem($produto->getCdfisopeitem());
        $itemProduto->setCdsufxcfoppr($produto->getCdsufxcfoppr());
        $itemProduto->setCdsitutribpr($produto->getCdsitutribpr());
        $itemProduto->setIdavisositem($produto->getIdavisositem());

        $itemProduto->setDtultatu(DateUtil::getDataAtual());
        $itemProduto->setNrorgultatu(Ambiente::getNrorg());
        $itemProduto->setCdoperultatu(Ambiente::getCdOperador());
        $itemProduto->setVrtotaitem($produto->getVrtotaitem());
        $itemProduto->setQtitemprod($produto->getQtitemprod());

        $itemProduto->persist();

        Registry::getEntityManager()->flush();

        return $itemProduto->getNrseqitprod();
    }

    public static function salvaRelacaoNotafiscalOrigem($lancamento, $lancamentoOrigem, $filialOrigem)
    {
        $itemnotafisc = new Itemnotafisc();

        $itemnotafisc->setCdfilial(Ambiente::getCdFilial());
        $itemnotafisc->setNrlanctonf($lancamento);
        $itemnotafisc->setNrsequitnota(self::geraProximoNRSEQUITNOTA(Ambiente::getCdFilial(), $lancamento));
        $itemnotafisc->setCdfilialit($filialOrigem);
        $itemnotafisc->setNrlanctonfit($lancamentoOrigem);
        $itemnotafisc->setNrorg(Ambiente::getNrorg());
        $itemnotafisc->setDtinclusao(DateUtil::getDataAtual());
        $itemnotafisc->setDtultatu(DateUtil::getDataAtual());
        $itemnotafisc->setNrorginclusao(Ambiente::getNrorg());
        $itemnotafisc->setCdoperinclusao(Ambiente::getCdOperador());
        $itemnotafisc->setNrorgultatu(Ambiente::getNrorg());
        $itemnotafisc->setCdoperultatu(Ambiente::getCdOperador());

        $itemnotafisc->persist();

        Registry::getEntityManager()->flush();
    }

    public static function geraProximoNRSEQUITNOTA($filial, $lancamento)
    {
        $nrsequitnota = Entities\Itemproduto::getRepository()->getMaxNRSEQUITNOTA($filial, $lancamento) + 1;

        return str_pad($nrsequitnota, 3, '0', STR_PAD_LEFT);
    }

    /**
     * @param $tipoOperacao
     * @param $itemproduto
     * @param $insercao
     */
    public static function informacoesComplementaresProduto(Tipooperac $tipoOperacao, Itemproduto $itemproduto,
                                                            $insercao)
    {
        $boDocuDevo = $tipoOperacao->getIdoperdevonf() == 'S';

        if (!$tipoOperacao->getIdatuesttpop()) {
            if ($tipoOperacao->getIdoperdevonf() == 'S' || $tipoOperacao->getCdtipooper() == '03') {
                Produto::openInfComplProd(
                    $itemproduto->getCdfilial(), DateUtil::getDataAtual(),
                    $itemproduto->getCdproduto(), $itemproduto->getCdalmoxarife(), $itemproduto->getCdlocalestoq(),
                    $itemproduto->getNrloteestq(), $itemproduto->getNrsublote(), $itemproduto->getNrorg(), false,
                    true, true, false, false, false, $boDocuDevo, false
                );
            } elseif ($insercao) {
                Produto::openInfComplProd(
                    $itemproduto->getCdfilial(), DateUtil::getDataAtual(),
                    $itemproduto->getCdproduto(), $itemproduto->getCdalmoxarife(), $itemproduto->getCdlocalestoq(),
                    $itemproduto->getNrloteestq(), $itemproduto->getNrsublote(), $itemproduto->getNrorg(), true,
                    true, true, false, false, true, $boDocuDevo, false
                );
            } else {
                Produto::openInfComplProd(
                    $itemproduto->getCdfilial(), DateUtil::getDataAtual(),
                    $itemproduto->getCdproduto(), $itemproduto->getCdalmoxarife(), $itemproduto->getCdlocalestoq(),
                    $itemproduto->getNrloteestq(), $itemproduto->getNrsublote(), $itemproduto->getNrorg(), false,
                    true, true, false, false, true, $boDocuDevo, false
                );
            }
        } else {
            if ($tipoOperacao->getIdoperdevonf() == 'S' || $tipoOperacao->getCdtipooper() == '03') {
                Produto::openInfComplProd(
                    $itemproduto->getCdfilial(), DateUtil::getDataAtual(),
                    $itemproduto->getCdproduto(), $itemproduto->getCdalmoxarife(), $itemproduto->getCdlocalestoq(),
                    $itemproduto->getNrloteestq(), $itemproduto->getNrsublote(), $itemproduto->getNrorg(), false,
                    true, true, false, true, false, $boDocuDevo, false
                );
            } elseif ($insercao) {
                Produto::openInfComplProd(
                    $itemproduto->getCdfilial(), DateUtil::getDataAtual(),
                    $itemproduto->getCdproduto(), $itemproduto->getCdalmoxarife(), $itemproduto->getCdlocalestoq(),
                    $itemproduto->getNrloteestq(), $itemproduto->getNrsublote(), $itemproduto->getNrorg(), true,
                    true, true, false, true, true, $boDocuDevo, false
                );
            } else {
                Produto::openInfComplProd(
                    $itemproduto->getCdfilial(), DateUtil::getDataAtual(),
                    $itemproduto->getCdproduto(), $itemproduto->getCdalmoxarife(), $itemproduto->getCdlocalestoq(),
                    $itemproduto->getNrloteestq(), $itemproduto->getNrsublote(), $itemproduto->getNrorg(), false,
                    true, true, false, true, true, $boDocuDevo, false
                );
            }
        }
    }

    public function beforeLancamento()
    {
    }

    public function afterLancamento()
    {
    }

    /**
     * Grava os impostos dos produtos e os impostos totais da nota.
     *
     * @author fernandomatias - 18/04/2012
     */
    public function gravaImpostosProduto()
    {
        $incidencias = $this->retornaIncidenciaProduto();
        foreach ($incidencias as $index => $incidencia) {
            $this->params['CDIMPOSTO'] = $incidencia['CDIMPOTEMP'];
            $aliquota                  = $this->retornaAliquotaProduto();
            if (is_null($aliquota[0]['VRALIQUTMP'])) {
                $this->params['VRALIQITEM'] = 0;
            } else {
                $this->params['VRALIQITEM'] = $aliquota[0]['VRALIQUTMP'];
            }
            $this->adicionaValoresImpostoProduto();
            $this->setImpostoTotal();
            if (empty($this->impostoTotal)) {
                $this->adicionaValoresImpostoTotal();
            }
        }
        ;
        $this->persistirItensImposto();
        $this->persistirImpostoTotal();
    }

    /**
     * Valida se o ducumento ja foi cadastrado.
     * @return bool
     */
    public function documentoJaCadastrado()
    {
        $id = array(
            'nrorg'      => Ambiente::getNrorg(),
            'cdforneced' => $this->notafiscal->getCdforneced());
        /* @var $parametros Paramforne */
        $parametros   = Paramforne::getObject($id);
        $nrdiasbloqnf = 0;
        if (!is_object($parametros) || is_null($parametros->getNrdiasbloqnffor())) {
            /* @var $paramGeral \Geral\Entities\Paramgeral */
            $paramGeral = Entities\Paramgeral::getObject(
                array(
                    'nrorg' => Ambiente::getNrorg())
            );
            if ($paramGeral->getNrdiasbloqnf()) {
                $nrdiasbloqnf = $paramGeral->getNrdiasbloqnf();
            }
        } else {
            $nrdiasbloqnf = $parametros->getNrdiasbloqnffor();
        }

        if ($nrdiasbloqnf != 0) {
            /* @var $notafiscal \Fiscal\Repositories\Notafiscal */
            $notafiscal = Notafiscal::getRepository();
            $nf         = $notafiscal->consultaNotaFiscalJaExiste(
                $this->notafiscal->getCdforneced(),
                $this->notafiscal->getSerieentr(), $this->notafiscal->getNrnotafisc(),
                $this->notafiscal->getCdtipooper(), Ambiente::getNrorg()
            );

            $data = new \Zend_Date($nf);
            $data->addDay($nrdiasbloqnf);

            $paramestoque = Entities\Paramestoque::getObject(
                array(
                    'nrorg' => Ambiente::getNrorg())
            );
            if ($data->isLater($paramestoque->getDtprocess())) {
                return true;
            }
        }
        return false;
    }
}


